#!/usr/bin/perl

use strict;
use utf8;

use Getopt::Long;
use Pod::Usage;

binmode(STDIN, ':utf8');
binmode(STDOUT, ':utf8');
binmode(STDERR, ':utf8');

my $verbose;
my $help;
my $man;

my $systemFile;
my $GSFile;
my $run;
my $Instr2Info = 0;

my %GSData;
my %systemData;

my %measureResults;

if (scalar(@ARGV) ==0) {
    $help = 1;
}

Getopt::Long::Configure ("bundling");

GetOptions('help|?'     => \$help,
	   'man|m'     => \$man,
	   'verbose|v'     => \$verbose,
	   'systemfile|s=s'     => \$systemFile,
	   'GSfile|g=s'     => \$GSFile,
	   'run|r=s' => \$run,
	   'instr2info|i' => \$Instr2Info,
    );

pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;

my %Classes;

warn "read GS\n" if ($verbose);
&readFile($GSFile, \%GSData, \%Classes, $verbose);
warn "read system\n" if ($verbose);
&readFile($systemFile, \%systemData, $verbose, $Instr2Info);


&makeEval(\%GSData, \%systemData, \%measureResults, $run, \%Classes, $verbose);

print "------------------------------------------------------------------------\n";

sub makeEval {
    my ($GSdata, $systmdata, $results, $r, $classes, $verb) = @_;
    my $id;
    my $class;
    my $macroPrec;
    my $sumPrec;
    my $sumRecall;
    my $prec;
    my $recall;
    # my %classes;
    my $class;

    foreach $id (keys %$GSdata) {
	# $classes{$GSdata->{$id}}++;
	if ((exists $systmdata->{$id}) && (exists $classes->{$systmdata->{$id}})) {
	    if ($GSdata->{$id} eq $systmdata->{$id}) {
		print "P($id) = 1\n" if ($verb);
		$results->{micro}->{TP}++;
		$results->{macro}->{$GSdata->{$id}}->{TP}++;
	    } else {
		print "P($id) = 0\n" if ($verb);
		$results->{micro}->{FP}++;
		$results->{micro}->{FN}++;
		$results->{macro}->{$systmdata->{$id}}->{FP}++;
		$results->{macro}->{$GSdata->{$id}}->{FN}++;
	    }
	} else {
	    warn "$id not in system file\n";
	    print "P($id) = 0\n" if ($verb);
	    $results->{micro}->{FP}++;
	    $results->{micro}->{FN}++;
	    foreach $class (keys %$classes) {
		warn "class: $class\n" if ($verb);
		$results->{macro}->{$class}->{FP}++;
		$results->{macro}->{$class}->{FN}++;
	    }
	}
    }
    my $out = 0;
    foreach $id (keys %$systmdata) {
	if (!exists $GSdata->{$id}) {
	    warn "$id not in GS file\n" if ($verb);
	    $out++;
	}
    }
    if ($out > 0) {
	warn "$out lines not taken into account\n" if ($verb);
    }
    print "Micro precision (run $r): " . ($results->{micro}->{TP} / ($results->{micro}->{TP} + $results->{micro}->{FP}))  . "\n";
    print "\tTP: " . ($results->{micro}->{TP} * 1) . "\n";
    print "\tFP: " . ($results->{micro}->{FP} * 1) . "\n";

    print "Per class\n";
    foreach $class (keys %$classes) {
	if (($results->{macro}->{$class}->{TP} != 0) || ($results->{macro}->{$class}->{FP} != 0)) {
	    $prec = $results->{macro}->{$class}->{TP} / ($results->{macro}->{$class}->{TP} + $results->{macro}->{$class}->{FP});
	} else {
	    $prec = 0;
	}
	if (($results->{macro}->{$class}->{TP} != 0) || ($results->{macro}->{$class}->{FN} != 0)) {
	    $recall = $results->{macro}->{$class}->{TP} / ($results->{macro}->{$class}->{TP} + $results->{macro}->{$class}->{FN});
	} else {
	    $recall = 0;
	}
	$sumPrec += $prec;
	$sumRecall += $prec;
	print "\tP($class): " . ($prec * 1)  . "\n";
	print "\t\tTP: " . ($results->{macro}->{$class}->{TP} * 1) . "\n";
	print "\t\tFP: " . ($results->{macro}->{$class}->{FP} * 1) . "\n";
	print "\t\tFN: " . ($results->{macro}->{$class}->{FN} * 1) . "\n";
    }
    print "Macro precision (run $r): " . ($sumPrec/scalar(keys %$classes)) . "\n";
    print "Macro recall (run $r): " . ($sumRecall/scalar(keys %$classes)) . "\n";
}

sub readFile {
    my ($file, $data, $classes, $instr2info) = @_;
    my @infos;
    my $line;
    my %convert = ('i' => "INFORMATION",
		   'o' => "OPINION",
		   's' => "SENTIMENT",
		   'e' => "EMOTION",
	);
    my $elt;

    open FILE, $file or die "No such file $file\n";
    while($line = <FILE>) {
	chomp $line;
	if ($line !~ /^\s*$/) {
	    @infos = split /\s+/, $line;
	    if (($instr2info) && ($infos[1] eq "INSTRUCTION")) {
		$infos[1] = "INFORMATION";
	    }
	    if (exists $convert{$infos[1]}) {
		warn $infos[1] . "=> " . $convert{$infos[1]} . "\n";
		$infos[1] = $convert{$infos[1]};
	    }
	    $data->{$infos[0]} = $infos[1];
	    if (defined $classes) {
		$classes->{$infos[1]}++;
	    }
	}
    }
    close FILE;
    warn "$file: " . scalar(keys(%$data)) . " lines read\n";
}


########################################################################

=head1 NAME

evalT1-T2.pl - Evaluation script for the tasks 1, 2.1 and 2.2 of DEFT2015


=head1 SYNOPSIS

evalT1-T2.pl [option] --systemfile <filename> --GSfile <filename> --run <string>

where option can be --help --man --verbose

=head1 OPTIONS AND ARGUMENTS

=over 4

=item --systemfile <filename>,  -s <filename>

Specification of the file containing the results issued from the participant system.

=item --GSfile <filename>, -g <filename>

Specification of the file containing the Gold Standard.

=item --run <string>, -r <string>

To specify the name of the run.


=item --help, -?

Print help message for using evalT1-T2.pl

=item --man, -m

Print man page of evalT1-T2.pl

=item --verbose, -v

Go into the verbose mode

=back

=head1 DESCRIPTION

This script computes the results for the task 1, 2.1 and 2.2 of DEFT
2015. It provides micro and macro-precision.

=head1 EXAMPLES

evalT1-T2.pl -g Test/T1.txt -s Test/T1-r1.txt -r run1

evalT1-T2.pl -g Test/T2.1.txt -s Test/T2.1-r2.txt -r run2

evalT1-T2.pl -g Test/T2.2.txt -s Test/T2.2-r2.txt -r run2

=head1 SEE ALSO

DEFT 2015 web site: <https://deft.limsi.fr/2015/>

=head1 AUTHOR

Thierry Hamon, E<lt>hamon@limsi.frE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2015 Thierry Hamon

This is free software; you can redistribute it and/or modify it under
the same terms as Perl itself, either Perl version 5.8.4 or, at your
option, any later version of Perl 5 you may have available.

=cut

